/**
 * \file   motors.c
 * \author Pierluigi Rinaldi (p.rinaldi@comune.livorno.it)
 * \version   V 0.1
 * \date   16 ottobre, 2013
 *
 * \brief  Driver pilotaggio motori PWM LAP.
 *
 * Funzioni di inizializzazione e pilotaggio motori in PWM LAP con PIC18F4431.
 */

#include <p18cxxx.h>
#include  "motors.h"

void initMotors(unsigned int period){

// Power Timer Control
	PTCON0 = 0b00000000;        	// 1:1 postscaler, 1:1 prescale, free running
	PTCON1 = 0b10000000;        	// PWM time base ON, count up
	PTPERH = (unsigned char) (period >> 8);
	PTPERL = (unsigned char) (period);

// PWM Control
	PWMCON0 = 0b00110000;        	// 011 = PWM0, PWM1, PWM2 and PWM3 I/O pins enabled for PWM output complementary
	PWMCON1 = 0b00000000;        	// 1:16 postscaler, updates abilitati, override asincrono

	DTCON = 0b00010000;        		// Quarzo=10Mhz; prescaler=Fosc/2 dead time 3.2uSec... 10000 = 32

// Output Override
	OVDCOND = 0b00111111;			// L'output  controllato dal DutyCycle e TimeBase
	OVDCONS = 0b00111111;			// Tutti i pin sono INACTIVE quando in override

// Fault Configuration
	FLTCONFIG = 0x00;				// Fault A e B disabilitati
           
	return ;
}

unsigned char setMotor(unsigned char motor, unsigned char enable,  int dc){
	unsigned int period;
	int dutycycle;                
	
// Controlla se i parametri sono corretti
	if ((motor < 0 ) || (motor > 2)) return MOTOR_NUMBER_ERROR;
	if ((dc > (PWM_LIMIT)) || (dc < -(PWM_LIMIT))) return MOTOR_PWR_ERROR;

//Gestisce il segnale Enable da inviare al driver di potenza.
 	if(enable){
	 	switch ((unsigned char) motor){
		 	case MOTOR_ALL :	EN_MOT1 = ENABLE;
		 						EN_MOT2 = ENABLE;
 								break;
		 	case MOTOR_1 :		EN_MOT1 = ENABLE;
 								break;
 			case MOTOR_2 :		EN_MOT2 = ENABLE;
 								break;
 		}
	}else{
	 	switch ((unsigned char) motor){
		 	case MOTOR_ALL :	EN_MOT1 = DISABLE;
		 						EN_MOT2 = DISABLE;
 								break;
		 	case MOTOR_1 :		EN_MOT1 = DISABLE;
 								break;
 			case MOTOR_2 :		EN_MOT2 = DISABLE;
 								break;
 		}
	}

// legge la frequenza di lavoro per calcolare la risoluzione
	period = ((unsigned int)(PTPERH) << 8) + PTPERL;

// calcola il dutycycle da applicare in base alla precisione
 	switch ((unsigned int) period){
	 	case PWM_10K : 	dutycycle = 2048 +  dc; 
	 					break;		// 12 bit (4096)
 		case PWM_20K : 	dutycycle = 1024 + (dc / 2); 
 						break;		// 11 bit (2048)
 		case PWM_40K : 	dutycycle = 512  + (dc / 4); 
 						break;		// 10 bit (1024)
	}

 // Aggiorna i registri relativi al motore interessato
 	switch (motor){
	 	case MOTOR_ALL : 	PDC0H = dutycycle >> 8 ; 
	 						PDC0L = dutycycle; 
 				 			PDC1H = dutycycle >> 8 ; 
 							PDC1L = dutycycle; 
 							break;
	 	case MOTOR_1 : 		PDC0H = dutycycle >> 8 ; 
	 						PDC0L = dutycycle; 
	 						break;
 		case MOTOR_2 :		PDC1H = dutycycle >> 8 ; 
 							PDC1L = dutycycle; 
 							break;
  	}
	return 0;
}
